@@ -9,6 +9,7 @@ import shlex
 import sys
 
 from twisted.cred import checkers, credentials
+from twisted.conch.checkers import SSHPublicKeyDatabase
 from twisted.conch.avatar import ConchUser
 from twisted.conch.ssh import common
 from twisted.conch.ssh.session import SSHSession, ISession, SSHSessionProcessProtocol
@@ -20,6 +21,7 @@ from twisted.internet.interfaces import IProcessTransport
 from twisted.internet.error import ProcessTerminated
 from twisted.python import components, log, failure
 from twisted.python.failure import Failure
+from twisted.conch.ssh.keys import Key
 from zope import interface
 
 
@@ -82,6 +84,15 @@ class PasswordChecker:
             self._cbPasswordMatch, str(credentials.username))
 
 
+class PublicKeyChecker(SSHPublicKeyDatabase):
+    def __init__(self, checker):
+        self.checker = checker
+
+    def checkKey(self, credentials):
+        return defer.maybeDeferred(self.checker, credentials.username,
+                                   credentials.blob)
+
+
 class VCSSession(SSHSession):
     def __init__(self, *args, **kw):
         SSHSession.__init__(self, *args, **kw)
@@ -222,6 +233,16 @@ class GitRealm:
 
 
 class BaseAuthnz:
+
+    def __init__(self, pkeys_file=None):
+        self.methods = {}
+        self.pkeys_file = pkeys_file
+        self.methods["password"] = True
+        self.methods["publick_key"] = False
+
+    def checker_registred(self, method):
+        return method[method]
+
     def can_read(self, username, gitpath):
         pass
 
@@ -231,6 +252,23 @@ class BaseAuthnz:
     def check_password(self, username, password):
         pass
 
+    def check_publickey(self, username, keyblob):
+        if self.pkeys_file:
+            try:
+                with open(self.pkeys_file, 'rb') as f:
+                    for line in f:
+                        try:
+                            user, key = line.split(':', 1)
+                            if (username == user.strip() and
+                                keyblob == Key.fromString(data=key.strip()
+                                                               ).blob()):
+                                return True
+                        except:
+                            log.err(None, "Loading key failed")
+            except:
+                log.err(None, 'No key file')
+        return False
+
 
 def create_factory(private_keys, public_keys, authnz_instanse, vcs_configuration):
     components.registerAdapter(GitSession, VCSConchUser, ISession)
@@ -241,6 +279,11 @@ def create_factory(private_keys, public_keys, authnz_instanse, vcs_configuration
         authnz = authnz_instanse
         vcs_config = VCSConfiguration(vcs_configuration)
         portal = Portal(GitRealm(authnz, vcs_config))
-        portal.registerChecker(PasswordChecker(authnz.check_password))
+        if hasattr(authnz, "check_password"):
+            log.msg("Registering PasswordChecker")
+            portal.registerChecker(PasswordChecker(authnz.check_password))
+        if hasattr(authnz, 'check_publickey'):
+            log.msg("Registering PublicKeyChecker")
+            portal.registerChecker(PublicKeyChecker(authnz.check_publickey))
 
     return VCSServer
\ No newline at end of file
